#!/usr/local/bin/perl
#
# Perl Script to edit a translation CSV file for the Mouse U74A array, provided by Affymetrix, into a format suitable for POSOC.
# This script requires two command line arguments: the name of the input file, and the desired name of the output file. This should have the file extension '.mtx2' and correspond to the transfile requested in the POSOC command line.
# Ian Grieve
# 14/07/05

use strict;
my $affymetrix_csv = "";
my $affymetrix_mtx2 = "";
if ($#ARGV < 1)
{
    die print "Please supply the command line arguments required\n";
}

else
{
    $affymetrix_csv = $ARGV[0];
    $affymetrix_mtx2 = $ARGV[1];
}

my $quotation = "'";
our $affyfile_input = $affymetrix_csv;
print $affyfile_input;
print "\n";
open (AFFYFILE, $affyfile_input) || die print "Affyfile Convertor cannot open $affyfile_input\n";
# The affyfile_input is a CSV file downloaded from the Affymetrix website.

our $intermediate_output = 'converter_intermediate.txt';
#Intermediate Output file.

our $transfile_output = $affymetrix_mtx2;
#Final Output file.

our $debugfile_output = 'converter_debug.mtx2';
#File containing Affy ID's  with no GO term associated


open (INTERMEDIATEFILE, ">$intermediate_output") || die print "Affyfile Convertor cannot open $intermediate_output\n";

open (TRANSFILE, ">$transfile_output") || die print "Affyfile Convertor cannot open $transfile_output\n";

open (DEBUGFILE, ">$debugfile_output") || die print "Affyfile Convertor cannot open $debugfile_output\n";

my @affyfile = <AFFYFILE>;
my $affyline = "";
my $probe_ID = "";
my $GO_biological_process = "";
my $GO_cellular_component = "";
my $GO_molecular_function = "";
my $GO_term_type = "";
my $GO_term = "";
my $GO_term_desc = "";
my $GO_term_evidence = "";

shift @affyfile;

foreach $affyline (@affyfile)
{
    chomp $affyline;

    if ($affyline =~ /"([\w\d\-\/]*)","[^"]*","[^"]*","[\w\d\s,]*","[^"]*","[\w\s]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","[^"]*","([^"]*)","([^"]*)","([^"]*)"/)
# Each line of the annotation file contains many columns, mostly irrelevant to the function of converting Affy ID's to GO terms. The bracketed 'columns' $2-4 are those containing biological process, cellular component and molecular function information respectively.

{
    print $1;
    print "\n";
    print INTERMEDIATEFILE $1;
    print INTERMEDIATEFILE "\t";
    $probe_ID = $1;

    $GO_biological_process = $2;

    $GO_biological_process =~ s/ \/\/\/ / GO:/g;
    $GO_biological_process =~ s/ \/\/ /;/g;

    print INTERMEDIATEFILE "Biological_Process";
    print INTERMEDIATEFILE "\t";
    print INTERMEDIATEFILE "GO:";
    print INTERMEDIATEFILE $GO_biological_process;
    print INTERMEDIATEFILE "\n";

    
    $GO_cellular_component = $3;
    $GO_cellular_component =~ s/ \/\/\/ / GO:/g;
    $GO_cellular_component =~ s/ \/\/ /;/g;
#    print INTERMEDIATEFILE "Cellular_Component";
#    print INTERMEDIATEFILE "\t";
#    print INTERMEDIATEFILE "GO:";
#    print INTERMEDIATEFILE $GO_cellular_component;
#    print INTERMEDIATEFILE "\n";
    
    $GO_molecular_function = $4;
    $GO_molecular_function =~ s/ \/\/\/ / GO:/g;
    $GO_molecular_function =~ s/ \/\/ /;/g;
#    print INTERMEDIATEFILE "Molecular_Function";
#    print INTERMEDIATEFILE "\t";
#    print INTERMEDIATEFILE "GO:";
#    print INTERMEDIATEFILE $GO_molecular_function;
#    print INTERMEDIATEFILE "\n";
}

else {print INTERMEDIATEFILE "ERROR!";}
}
close (INTERMEDIATEFILE);

our $intermediate_input = 'converter_intermediate.txt';
open (INTERMEDIATEARRAY, $intermediate_input) || die print "Affyfile Convertor cannot open $intermediate_input\n";

my @intermediatefile = <INTERMEDIATEARRAY>;
my $intermediateline = "";

foreach $intermediateline (@intermediatefile)
{
    if ($intermediateline !~ /-{3}/)
# Removes lines containing no GO terms, as these cause StringIndexOutOfBounds errors in the POSOC translator module.
    {
	print TRANSFILE $intermediateline;
    }

    else
    {
	print DEBUGFILE $intermediateline;
	print DEBUGFILE "\n";
# This DEBUGFILE contains the lines from TRANSFILE that contained no GO term.
    }
}

close (TRANSFILE);
close (DEBUGFILE);
close (INTERMEDIATEARRAY);
unlink ('converter_intermediate.txt') || die print "Affyfile Converter cannot unlink converter_intermediate.txt";
unlink ('converter_debug.mtx2') || die print "Affyfile Converter cannot unlink converter_debug.mtx2";
	



